#!/bin/env python
# -*- coding: UTF-8 -*-
## \namespace comunicaciones
# \brief Conjunto de clases necesarias para implementar la comunicacion entre procesos a traves de red.

import os
import signal
import time
import socket
import struct
import re

##\class EBC
#\brief Clase basica de comunicaciones
# EBC (Elemento Basico de Comunicacion)
#\version 1.2.2-1
#\date 16-11-2007
class EBC:
	## \fn __init__(self)
	# \brief Constructor de la clase EBC. Crea un socket y fija las opciones iniciales
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
	
	## \fn __del__(self)
	# \brief Destructor de la clase EBC. Cierra el socket de la instancia
	# \param[in] self Instancia de la clase que realiza la llamada
	def __del__(self):
		self.sock.close()

	## \fn cerrar_socket(self)
	# \brief Cierra el socket de la instancia
	# \param[in] self Instancia de la clase que realiza la llamada
	def cerrar_socket(self):
		self.sock.close()

##\class EBCr
#\brief Clase de comunicaciones para recepcion
# EBCr (Elemento Basico de Comunicacion de recepcion)
#\version 1.2.2-1
#\date 16-11-2007
class EBCr(EBC):
	## \fn __init__(self)
	# \brief Constructor de la clase EBCr, llama al constructor de la clase padre
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		EBC.__init__(self)
	
	## \fn __del__(self)
	# \brief Constructor de la clase EBCr, llama al constructor de la clase padre
	# \param[in] self Instancia de la clase que realiza la llamada
	def __del__(self):
		EBC.__del__(self)
	
	## \fn abrir_socket_puerto(self, puerto)
	# \brief Funcion para asociar un socket a un puerto y mantenerlo a la escucha
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] puerto Puerto al que se asocia el socket
	def abrir_socket_puerto(self, puerto):
		self.sock.bind(('', puerto))
		self.sock.listen(20)

	## \fn leer_EBCr(self)
	# \brief Funcion para recepcionar mensajes desde un emisor
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Mensaje recepcionado
	# \return Direccion IP del emisor
	# \return Direccion MAC del emisor
	def leer_EBCr(self):
		cliente, addr = self.sock.accept()
		mensaje = cliente.recv(1024)
		cliente.close()
		mac = self.obtener_dir_mac(addr[0])
		return mensaje, addr[0], mac

	## \fn obtener_dir_mac(self, ip)
	# \brief Funcion para obtener la direccion MAC a partir de una direccion IP
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion IP
	# \return Direccion MAC correspondiente a la direccion IP de entrada
	def obtener_dir_mac(self, ip):
		mac = ''
		fichero_tabla_arp = open('/proc/net/arp', 'r')
		tabla_arp = fichero_tabla_arp.readlines()
		for direccion in tabla_arp:
			datos_direccion = re.split('\s+', direccion)
			if datos_direccion[0] == ip:
				return datos_direccion[3]
		return mac

##\class EBCe
#\brief Clase de comunicaciones para envio
# EBCe (Elemento Basico de Cominicacion de envio)
#\version 1.2.2-1
#\date 16-11-2007
class EBCe(EBC):
	## \fn __init__(self)
	# \brief Constructor de la clase EBCe, llama al constructor de la clase padre
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		EBC.__init__(self)
		self.sock.settimeout(1.2)

	## \fn __del__(self)
	# \brief Destructor de la clase EBCe, llama al destructor de la clase padre
	# \param[in] self Instancia de la clase que realiza la llamada
	def __del__(self):
		EBC.__del__(self)

	## \fn enviar_mensaje_EBCe(self, ip, mensaje, puerto)
	# \brief Funcion para enviar un mensaje a un cliente y puerto concreto
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion IP del cliente al que hay que enviar el mensaje
	# \param[in] mensaje Mensaje que se va a enviar(cadena de caracteres)
	# \param[in] puerto Puerto utilizado para el envio del mensaje
	# \return Codigo de finalizacion de la operacion
	# \retval 1 La operacion se ha realizado incorrectamente
	# \retval 0 La operacion se ha realizado correctamente
	# \exception socket.error Error de comunicacion
	def enviar_mensaje_EBCe(self, ip, mensaje, puerto):
		# conexion con el receptor
		try:
			self.sock.connect((ip, puerto))
		except socket.error, msg:
			print "Error de conexion: %s" % msg
			self.sock.close()
			return (1)
		# envio del mensaje
		self.sock.send(mensaje)
		#try:
		#	entrante = self.sock.recv(1024)
		#except socket.error, msg:
		#	print "Error de conexion: %s" % msg
		#	self.sock.close()
		#	return (1)
		self.sock.close()
		return (0)

	##\fn conectar_socket(self, ip, puerto)
	# \brief Funcion para conectarse a una direcion IP y un puerto remoto
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion IP del cliente al que se quiere conectar
	# \param[in] puerto Puerto utilizado para realizar la conexion remota
	# \return Codigo de finalizacion de la operacion
	# \retval 1 La operacion se ha realizado incorrectamente
	# \retval 0 La operacion se ha realizado correctamente
	# \exception socket.error Error de comunicacion
	def conectar_socket(self, ip, puerto):
		try:
			self.sock.connect((ip, puerto))
		except socket.error, msg:
			print "Error de conexion: %s" % msg
			return (1)
		return (0)

	##\fn cerrar_conexion(self)
	# \brief Funcion para cerrar el socket de conexion
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def cerrar_conexion(self):
		self.sock.close()
